# Communication LRS

We have the problem with how to communicate between different agents
when we most probably have unreliable or band width limited
communication channels between agents.

We can choose one of the following way to communicate between agents:

- Use ROS2. We need to make sure the DDS layer is configured correctly
in this case. All agents will have the same value for
ROS_DOMAIN_ID. THIS IS NOT CURRENTLY WORKING.
- Use ZENOH. We specify what should be communicated between agents.  All agents
have different ROS_DOMAIN_ID or ROS_LOCALHOST_ONLY set to 1.
- Use JSON coding of messages and send the encoded message between agents using an MQTT
  broker. All agents will have different value for ROS_DOMAIN_ID.

We want to minimize the regular communication between agents.  One
special problem is /tf so we will soon change the code here to use /ns/tf for the tf.


## Issues

- Lossy communication. Protocols to detect and handle.
- Some topics are used by other agents.  One issue is if such topics
  should have a specific prefix like "/ext". Now it does not have that.
- configuration stride, tf...


## ROS2

Set the environment variable TST_REMOTE_TRANSPORT to ros.

No topics should be sent regularly between agents using ROS2.

This is currently not implemented.

## ZENOH

Set the environment variable TST_REMOTE_TRANSPORT to zenoh.

## JSON and MQTT Broker

This still works but we now prefer to not send any data this
way. Instead the main Wara-ps broker is used to forward some data
between agents.

Each agent must have a unique ROS_DOMAIN_ID. This can be impractical...

When using an MQTT broker to communicate between ROS2 agents (and also
between ROS1 and ROS2 agents) there are two programs running on the
ROS2 agent:

- **mqtt_to_ros**: This programs connect to the broker and decode the
  JSON encoded ROS messages it receives from the MQTT topics and push
  them out on ROS topics.
- **ros_to_mqtt**: This program listen to ROS2 messages and push them
  on the relevant MQTT topic as JSON strings.  The topics they listen
  to are prepended with the prefix "/ext". That means that all
  programs generating messages that is also intended for external
  agents need to prepend "/ext" to the topic. The **mqtt_to_ros**
  program will push out the message on the topic without a prefix (except for /ext/tf).
  This program can also be configured to reduce the number of messages
  it sends by having a stride for each topic. Also for each topic we
  can choose to send it to ALL agents or to a specific agent and we
  can also choose to send it on channel a or channel b. In the ROS1
  legacy system channel a is LTE and channel b is WiFi.

The MQTT topic is built up in the following way:

- roscombridge
- sender
- channel
- ext
- ROS topic
- receiver : ALL or namespace for unit/agent

For example:

- roscombridge/dji0/a/ext/unit_info/ALL
- roscombridge/dji0/a/ext/tf/op0

By giving "external:=true" to launch files we will tell the programs
that messages that needs to be forwarded will be put on a topic with
the prefix /ext.  So if this is given to the start of wdb then we will push to:
- /ext/unit_info
- /ext/unit_info_slow

The **ros_to_mqtt** program will put this on MQTT topics and the **mqt_to_ros** program will outpuy the messages on the topics /unit_info and /unit_info_slow.

For now see m300.launch.py were the external flag is set for the nodes:
- resource_agent
- tst_exec_factory
- wdb
- tst_expansion

<!---

## Unreliable Communication

Messages not arriving can in some cases be handled on the application level.

For example to distribute a TST tree to all possible agents we have a service call
TSTDistributeTreeFromUuid.srv:
```bash
string[] units      # List of units to distribute to.
                    # If empty distribute to everybody in possible units
string uuid         # Root node node_uuid of the TST to distribute
---
string com_uuid     # Used to check if tree have been distributed.
bool success
int32 error
string reason
```

And GetResponse.srv:
```bash
string cuuid        # Identifier for the thing we need an response for
---
bool have_response  # True if there is a response
string name
string[] need_response_from # List of units we need response from
string[] have_response_from # List of units we have response from
bool success        # If true the thing was OK.
```

So we make a service call to distribute the tree and get com_uuid as
response. We use com_uuid in the GetResponse service call to check if
we have gotten a response from every receiver of the tree.

The actual distribution of the tree is done by sending a TSTDistributeTreeInfo.msg message on a topic /distribute_tree:
```bash
string uuid         # Use this uuid for confirmation that the tree have been distributed
string sender       # Namespace for the sender
string tree_json    # The TST tree to distribute
```

And the message confirming reception Response.msg on topic /response:
```bash
string uuid
string cuuid        # uuid for the command this is a response to
string responder    # Namespace for the responder.
bool success        # If true the command was OK.
string fail_reason  # If fail, this is if given the reason.
```


## UNSTRUCTERED MISC

CURRENTLY UNSTRUCTURED NOTES BELOW, DO NOT READ!

### Agent Topics

- /unit_info
- /unit_info_slow
- /resources_info
- /visualization_marker
- /nsgeopose


### TST Topics

- /distribute_tree
- /update_node
- /response


### tf

How should tf be handles

### JSON forwarding using MQTT

- IDEA: Have a prefix for what is published: maybe /json, add in mqtt who it is from. /dji0/distribute_tree
- No prefix for locally distributed
- can that be made compatible with ROS1...



### com-uuid lists

Some service calls return a list of com-uuid which can be checked for
if the message they correspons to arrived properly.

there is one uuid per "data". So one node distributed to 5 agents will
give on com-uuid.  There is a service call to check that this com-uuid
have received confirmation from all 5 agents then.

### DistributeTree

- Uses com-uuid
- But unclear what happens if it fails
- Now I think it just fails and no resending

### DistributeNode changes

- Check if they use com-uuid

### AbortTree

- The distribution of abort-when-starting does not have any com-uuids

- But the sending of the abort signal uses com-uuid

- Should com-uuid be uses at all here?

--->
