#!/usr/bin/python3

import json
import sys
from os import listdir
from os.path import isfile, join

from optparse import OptionParser
parser = OptionParser()
parser.add_option ("", "--prototypes", action="store_true", dest="prototypes", help="Include prototypes.")
(options, args) = parser.parse_args()


def gen_markdown(obj):
    print("## {}".format(obj["key"]))
    print()
    print("**Description**: {}".format(obj["desc"]))
    print()

    if "tags" not in obj or len(obj["tags"]) == 0:
        print("**Tags**: None")
    else:
        print("**Tags**: {}".format(" ".join(obj["tags"])))

    print()

    if "params" in obj:    
        if len(obj["params"]) == 0:
            print("**Parameters**: None")

        else:
            print("**Parameters**:")
            print()
            #    print "PARAMS:", obj["params"]
            for p in obj["params"]:
                desc = ""
                if "desc" in p.keys():
                    desc = p["desc"]
                    print("- **{}** ({}): {}".format(p["key"], p["type"], desc))
        print()

    if "tst-params" in obj:
        if len(obj["tst-params"]) == 0:
            print("**TST Parameters**: None")

        else:
            print("**TST Parameters**:")
            print()
            #    print "PARAMS:", obj["params"]
            for p in obj["tst-params"]:
                desc = ""
                if "desc" in p.keys():
                    desc = p["desc"]
                    print("- **{}** ({}): {}".format(p["key"], p["type"], desc))
        print()

    if "task-params" in obj:            
        if len(obj["task-params"]) == 0:
            print("**Task Parameters**: None")

        else:
            print("**Task Parameters**:")
            print()
            #    print "PARAMS:", obj["params"]
            for p in obj["task-params"]:
                desc = ""
                if "desc" in p.keys():
                    desc = p["desc"]
                    print("- **{}** ({}): {}".format(p["key"], p["type"], desc))
        print()

    if "signals" in obj:
        if len(obj["signals"]) == 0:
            print("**Signals**: None")

        else:
            print("**Signals**:")
            print()
            #    print "SIGNALS:", obj["signals"]
            for p in obj["signals"]:
                if "desc" in p.keys():
                    name = p["key"]
                    desc = p["desc"]
                    print("- **{}**: {}".format(name, desc))
        print()
    print("**Node type**: {}".format(obj["node-type"]))
    print()
    req = " ".join(obj["required-params-for-execution"])
    print("**Required params for execution**: {}".format(req or "None"))
    print()
    print("**Node resources**: {}".format(" ".join(obj["node-resources"]) or "None"))
    print()

def gen_markdown_from_files(jsonfiles, type, status):    
    for f in sorted(jsonfiles):
        fullfile = f'{mypath}/{f}'
        print(f'FILE: {fullfile}', file=sys.stderr)
        jsonstr = open(fullfile, "r").read()
#        print "JSON:", jsonstr
        obj = json.loads(jsonstr)
        if obj["type"] != "tst node":
            continue
        if obj["node-type"] != type:
            continue
        if obj["status"] != status:
            continue
#        print f, obj["type"]
        gen_markdown(obj)


if __name__ == '__main__':
    mypath = "../lrs_exec/config"
    files = [f for f in listdir(mypath) if isfile(join(mypath, f))]
    jsonfiles = [f for f in files if f.endswith(".json")]
    print("""
# General Information

This page documents all currently available TST nodes.  It is
generated directly from the node specifications, and should serve as
the most up-to-date specification of nodes and their parameters.

Note that "Node resources" specifies the set of resource semaphores that
are automatically locked by the Delegation Framework during the
execution of a TST node. This is used to ensure that nodes that
interfere with each other cannot be executed concurrently.
    """)
    print("# Non-terminal TST node types")
    gen_markdown_from_files(jsonfiles, "non-terminal", "released")
    print("# Terminal TST node types"    )
    gen_markdown_from_files(jsonfiles, "terminal", "released")
    if options.prototypes:
        print("# Prototype TST node types (do not trust that these will be stable or permament)")
        gen_markdown_from_files(jsonfiles, "terminal", "prototype")
                 
