
# General Information

This page documents all currently available TST nodes.  It is
generated directly from the node specifications, and should serve as
the most up-to-date specification of nodes and their parameters.

Note that "Node resources" specifies the set of resource semaphores that
are automatically locked by the Delegation Framework during the
execution of a TST node. This is used to ensure that nodes that
interfere with each other cannot be executed concurrently.
    
# Non-terminal TST node types
## conc

**Description**: Concurrent node:  Immediately starts the execution of all children in parallel. This node cannot be aborted (abort its children instead).

**Tags**: control

**Parameters**: None

**Node type**: non-terminal

**Required params for execution**: None

**Node resources**: None

## seq

**Description**: Causes the children of this node to be executed in sequence. This node cannot be aborted (abort its children instead).

**Tags**: control

**Parameters**: None

**Node type**: non-terminal

**Required params for execution**: None

**Node resources**: None

## supp

**Description**: Work as a concurrent node. Is uses to indicate that all things under the node is support things which we need to send a finish to before exiting the node. Should be a child of a conc. The conc node and supp node have to send the finish signal.  Immediately starts the execution of all children in parallel. This node cannot be aborted (abort its children instead). IMPORTANT: There can not be a supp node under another supp node.

**Tags**: control

**Parameters**: None

**Node type**: non-terminal

**Required params for execution**: None

**Node resources**: None

## test-if

**Description**: A conditional node ('if' statement).  This node first executes the first child, which tests a condition (for example 'in-air-test' or 'query').  If the test is true, the second child is executed. If the condition is false, the third child is executed.  If the condition is unknown, the fourth child is executed.  Some of these children may be of type 'noop' (do nothing), 'fail' or 'abort'.  This node cannot be aborted.

**Tags**: control

**Parameters**: None

**Node type**: non-terminal

**Required params for execution**: None

**Node resources**: None

## try

**Description**: This node corresponds to a try-catch construct in many programming languages.  It first executes its first child.  If the first child fails, it then executes the second child, which can provide context-sensitive error handling.  If the first child was aborted, the third child is executed instead.  This node cannot be aborted.

**Tags**: control

**Parameters**:

- **try-success** (bool): Not to be specified from from the outside.  After the first node have been executed, the executor sets this to true if the first child succeeded.  This way, the TST can be inspected after or during execution.

**Node type**: non-terminal

**Required params for execution**: None

**Node resources**: None

# Terminal TST node types
## climb

**Description**: Climbs to a specified altitude.

**Tags**: vehicle

**Parameters**:

- **z** (float64): This specifies the desired elevation above the WGS84 ellipsoid (unless the height-above-takeoff value is specified).
- **height-above-takeoff** (float64): If specified, then the this value is interpreted as relative to the last point where the aircraft took off.  This allows missions to be specified in a more generic way without having to customize them for the altitude of a particular location.
- **height-above-ground** (float64): If specified, then the this value is interpreted as relative to the ground (or what a sensor in the UAV consider to be ground).  This allows missions to be specified in a more generic way without having to customize them for the altitude of a particular location.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly climb

## fly-path

**Description**: This node visits a sequence of waypoints.  Unlike fly-pattern, there is no constraint on exactly how the aircraft should fly between these waypoints.  As the node does not specify the expected starting position, the delegation process will not be able to determine in advance how far the aircraft must fly.  Therefore it will also not be able to compute a duration for the command.

**Tags**: vehicle

**Parameters**:

- **commanded-speed** (float64): The commanded (desired) flight speed in meters per second.  This can be omitted, in which case a platform-specific default speed will be used.  Do not use both commanded-speed and speed.
- **speed** (string): This allows the specification of a qualitative speed level as an alternative to commanding a specific speed.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.  Do not use both commanded-speed and speed.
- **follow-ground-flag** (bool): Fly at a given altitude above the ground, as specified by follow-ground-altitude.  The platform must have access to suitable altitude maps.  Currently not supported on the DJI Matrice.
- **follow-ground-altitude** (float64): See follow-ground-flag
- **waypoints** (geopoints): A sequence of way points to fly through or to.
- **path** (geopoints): Used in proposal and execution. A sequence of way points to fly through or to.
- **do-not-yaw-flag** (bool): Currently not supported on the DJI Matrice.
- **continue-resets-gimbal-flag** (bool): When continue reset the gimbal to the value it had when starting the executor.
- **wait-for-airspace-booking** (bool): Wait for airspace booking. If false fails if airspace could not be booked. Default value is true.
- **max-wait-time** (int32): Max time to wait for airspace booking. If value is <=0 then wait forever. Default value is 0

**Node type**: terminal

**Required params for execution**: waypoints

**Node resources**: fly fly_path

## fly-pattern

**Description**: This node flies a pattern, for example a search or scan pattern. Different type of patterns can be specified.  In contrast to fly-path, this node assumes that the exact trajectory is important.  For example, one may have to fly straight lines between given waypoints in order to gather suitably overlapping 3D mapping data..

**Tags**: vehicle

**Parameters**:

- **commanded-speed** (float64): The commanded (desired) flight speed in meters per second.  This can be omitted, in which case a platform-specific default speed will be used.  Do not use both commanded-speed and speed.
- **speed** (string): This allows the specification of a qualitative speed level as an alternative to commanding a specific speed.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.  Do not use both commanded-speed and speed.
- **follow-ground-flag** (bool): Fly at a given altitude above the ground, as specified by follow-ground-altitude.  The platform must have access to suitable altitude maps.  Currently not supported on the DJI Matrice.
- **follow-ground-altitude** (float64): See follow-ground-flag
- **continue-resets-gimbal-flag** (bool): When continue reset the gimbal to the value it had when starting the executor.
- **pattern-type** (string): The type of pattern to fly. Possible values: 'segments'.  (This node is new; additional patterns will be added.)
- **waypoints** (geopoints): If pattern-type is 'segments': A sequence of n waypoints, defining n/2 segments to fly.  Each segment must be flown in a straight line.  Aircraft may fly arbitrarily between one segment and the next.
- **path** (geopoints): Used in proposal and execution. A sequence of way points to fly through or to.
- **any-order-flag** (bool): If set, then the segments can be flown in arbitrary order.  Otherwise, they must be flown in the order they are specified.  Allowing segments to be flown in any order allows the aircraft to optimize its trajectory according to its flight envelope and starting position, but can interfere with certain forms of information gathering.
- **do-not-yaw-flag** (bool): Currently not supported on the DJI Matrice
- **wait-for-airspace-booking** (bool): Wait for airspace booking. If false fails if airspace could not be booked. Default value is true.
- **max-wait-time** (int32): Max time to wait for airspace booking. If value is <=0 then wait forever. Default value i 0

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly fly_pattern

## fly-to

**Description**: This node flies to a given position.  As it does not specify the expected starting position, the delegation process will not be able to determine in advance how far the aircraft must fly (unlike fly-from-to).  Therefore it will also not be able to compute a duration for the command.

**Tags**: vehicle

**TST Parameters**:

- **positions** (geopoints): Fly to one of the given positions.
- **named-position** (string): Position available in param server. The position is given in x,y,z world coordinates or in latitude, longitude, altitude GeoPoint conventions.
- **ref-p** (string): Reference to the position to fly to. Note that this position can be available on another unit. The named-position argument refers to somehing availble on the current unit.
- **ref-ns** (string): Namespace for the unit having the ref-p. If the empty string than the current unit is assumed.
- **control-mode** (string): Currently not supported on the DJI Matrice
- **mission-mode-flag** (bool): Use the aircraft's built-in mission execution mode to upload a mission corresponding to this TST node and execute it completely on board the aircraft.  If not specified, the aircraft can be controlled directly by the executor instead.
- **random-flag** (bool): Fly randomly.  This is used internally to test airspace booking.

**Task Parameters**:

- **p** (geopoint): The position to fly to
- **path** (geopoints): Used in proposal and execution. A sequence of way points to fly through or to.
- **commanded-speed** (float64): The commanded (desired) flight speed in meters per second.  This can be omitted, in which case a platform-specific default speed will be used.  Do not use both commanded-speed and speed.
- **speed** (string): This allows the specification of a qualitative speed level as an alternative to commanding a specific speed.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.  Do not use both commanded-speed and speed.
- **do-not-yaw-flag** (bool): Currently not supported on the DJI Matrice
- **use-current-altitude** (bool): If set to true use the current altitude for the fly to position.
- **wait-for-airspace-booking** (bool): Wait for airspace booking. If false fails if airspace could not be booked. Default value is true.
- **max-wait-time** (int32): Max time to wait for airspace booking. If value is <=0 then wait forever. Default value is 0
- **follow-ground-flag** (bool): Fly at a given altitude above the ground, as specified by follow-ground-altitude.  The platform must have access to suitable altitude maps.  Currently not supported on the DJI Matrice.
- **follow-ground-altitude** (float64): See follow-ground-flag
- **continue-resets-gimbal-flag** (bool): When continue reset the gimbal to the value it had when starting the executor.
- **fly-home-flag** (bool): If true fly to the home lat, lon using the specified altitude in fly-home-height.
- **fly-home-height** (float64): Height above takeoff point when doing fly-home
- **fly-straight-line-flag** (bool): If true, the aircraft must fly in a straight line from its current location to the target position.  Otherwise, there is no constraint on exactly how the aircraft should fly between these waypoints.  Currently not supported on the DJI Matrice.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly fly_to

## go-home

**Description**: Go back to a home position and land (or corresponding for non-flying things). Use a vehicle dependent default speed if no speed is specified.

**Tags**: None

**Task Parameters**:

- **speed** (string): This allows the specification of a qualitative speed level as an alternative to commanding a specific speed.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## in-air-goal

**Description**: Similar to take-off, but can expand in a platform-specific way to ensure that a platform is in the air.  For example, some platforms require assistance from human operators in order to take off, in which case this node can expand to use a general tell-operator node that requests operator assistance.

**Tags**: None

**Parameters**:

- **tell-operator-auto-confirm-flag** (bool): If this flag is set, the operator is assumed to agree to assisting in any tasks required for takeoff.  By default, the operator is always asked in advance, during the delegation phase.
- **height-above-takeoff** (float64): If specified, then climb to the specified altitude after taking off (the value is relative to the take-off position).  Otherwise, the altitude after take-off is unspecified and platform-specific.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly climb

## in-air-test

**Description**: Tests whether the platform is currently in the air.  This can be used as a test in conditional nodes.

**Tags**: query

**Parameters**:

- **answer** (string): Possible answers are yes, no, unknown

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## land

**Description**: Causes the platform to land.

**Tags**: vehicle

**Task Parameters**:

- **p** (geopoint): The position at which the platform should land, if land-at-current-position-flag is false.
- **land-at-current-position-flag** (bool): If this flag is true, land at the current position of the aircraft.  Otherwise, land at the position given by p
- **z** (float64): The offset from the default landing altitude to use in simulations. Default value is 0.0
- **heading** (float64): Specify this for fixed-wing platforms that require a heading in order to land.  Helicopters may ignore this parameter.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly

## look-at-bearing

**Description**: This task causes the platform with a camera to point that camera using the specified bearing. The task is intended for cases where one should for example capture video, so rather than pointing once, it encapsulates a process where the camera is continuously adjusted.  The task will only terminate when a signal $enough is sent to the task.

**Tags**: payload

**TST Parameters**: None

**Task Parameters**:

- **bearing** (float64): Bearing angle in degree. Positive is to the right (clockwise). If absoulte bearing is used zero degre is north
- **use-absolute-bearing** (bool): If true then bearing angle is relative to north.  Otherwise it is relative to the vehicles direction.
- **tilt** (float64): Tilt angle in degree. Positive is up.
- **tilt-body-relative** (bool): If true then the tilt angle is relative to the body of the vehicle. Otherwise it is relative to the world (horisontal plane).  If this flag is false and the camera cannot be controlled relative to the horisontal plane than act like the parameter was true.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: look_at_angle gimbal

## look-at-position

**Description**: The agent will try to observe a position by moving the camera (it is only allowed to move the camera). It will try to observe it until it gets an $enough signal. It will send back status in the feedback message telling if the position is in view.  Possible values for status relative to that: in-view-center, in-view, not-in-view.

**Tags**: None

**TST Parameters**: None

**Task Parameters**:

- **geopoint** (geopoint): The position to observe.
- **named-position** (string): If defined look at this position (that can change). If the string start with / then it is the position of that unit that is used instead.

**Signals**: None

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## look-at-positions

**Description**: The agent will try to observe a position or a set of positions by moving the camera (it is only allowed to move the camera). It will try to observe it until it gets an $enough signal. It will send back status in the feedback message telling if the position is in view.  Possible values for status relative to that: in-view-center, in-view, not-in-view. If more than one position is given as argument then the position nearest to the vehicle should be selected.

**Tags**: None

**TST Parameters**: None

**Task Parameters**:

- **geopoints** (geopoints): The positions to observe.
- **named-positions** (strings): If defined look at this position (that can change). If the string start with / then it is the position of that unit that is used instead.

**Signals**: None

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## move-path

**Description**: This node moves the agent along a path. It is allowed to prepare for moving inside the node, for example doing take-off before flying.  Also the specified altitudes is the minimal altitude, the flying should find postiions higher if the position is occupied with obstacles.

**Tags**: vehicle

**TST Parameters**: None

**Task Parameters**:

- **waypoints** (geopoints): The positions to move to
- **speed** (string): Qualitative speed level.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## move-to

**Description**: This node moves the agent to a specific position. It is allowed to prepare for moving inside the node, for example doing take-off before flying. Also the specified altitude is the minimal altitude, the flying should find postiions higher if the position is occupied with obstacles.

**Tags**: vehicle

**TST Parameters**: None

**Task Parameters**:

- **waypoint** (geopoint): The position to move to
- **speed** (string): Qualitative speed level.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.
- **camera-data-recording** (bool): If set save data from camera while moving.
- **laser-data-recording** (bool): If set save data from camera while moving.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## no-op

**Description**: This node does nothing, except that it immediately succeeds.  This can be useful for certain branches of a 'test-if' node.

**Tags**: None

**Parameters**:


**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## observe-position

**Description**: The agent will try to observe a given position. It will observe it until it gets an $enough signal.

**Tags**: None

**TST Parameters**:

- **commanded-speed** (float64): If the commanded-speed parameter is specified use this value for speed.
- **named-position** (string): A named positions. The position to observe. The position is given in x,y,z world coordinates or in latitude, longitude, altitude GeoPoint conventions.
- **replacement-wait-uuid** (string): uuid for the wait-signal node that waits for the replacement agent to be in place. If the value is the empty string no replacement signal is sent.

**Task Parameters**:

- **geopoint** (geopoint): The position to observe.
- **speed** (string): Qualitative specification of speed. The meaning is platform dependent. Possible valuess: fast, standard, slow
- **distance** (float64): If given or if greater than 0.0 then use this radius for circling the obervation position for vehicles that needs to move while observing the position. For vehicles that can hover uses this is the distance from which to observe.
- **height** (float64): If vehicles can hover this is the wanted height above the position to observe.
- **duration** (int32): Time to observe the position in seconds. If negative never stop and an $enough signal have to be sent to finish the TST execution.

**Signals**:

- **$pause**: Pause what you are doing (as soon as you can), release resources for a period of time so you can do something else for a while.
- **$continue**:  Resume what you were doing, get back on track.
- **$enough**: The ordinary meaning -- the entire observe-pos is done, regardless of how much it actually did (can $enough in the middle of climb => do nothing more).
- **height-ok**: Tells the agent it is already at the correct height relative ti the object it is supposed to observe.  No need to continue climbing. Continue with getting nearer to the object to observe.
- **near-enough**: Tells the agent it is already sufficiently close to the object it is supposed to observe.  No need to continue flying or navigating closer. Continue with actually observing the position.
- **stop-countdown**: Pause counting down the timer.  If the timer hasn't started yet, that's fine: Should set a flag that the timer should not be running
- **continue-countdown**: 

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## return-home

**Description**: Return a vehicle to a home position. Use a vehicle dependent default speed if no speed is speicied.

**Tags**: None

**Parameters**:

- **commanded-speed** (float64): The commanded (desired) flight speed in meters per second.  This can be omitted, in which case a platform-specific default speed will be used.  Do not use both commanded-speed and speed.
- **speed** (string): This allows the specification of a qualitative speed level as an alternative to commanding a specific speed.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.  Do not use both commanded-speed and speed.
- **wait-for-airspace-booking** (bool): Wait for airspace booking. If false fails if airspace could not be booked. Default value is true.
- **max-wait-time** (int32): Max time to wait for airspace booking. If value is <=0 then wait forever. Default value i 0

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## search-area

**Description**: This node specifies a search of an area given by a list of geopoints. The area is 'closed' by assuming a segment between the last and first position in the list of geopoints. It is allowed to prepare for moving inside the node, for example doing take-off before flying.

**Tags**: vehicle

**TST Parameters**: None

**Task Parameters**:

- **area** (geopoints): The area to search.
- **speed** (string): Qualitative speed level.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.
- **target-type** (string): The type of the search target. Possible values: person, boat, ship, car, ...
- **area-type** (string): The type of the search target. Possible values: water, beach, forrest, field
- **target-size** (float64): The estimates size of the search target in meter.
- **camera-data-recording** (bool): If set save data from camera while moving.
- **laser-data-recording** (bool): If set save data from camera while moving.
- **go-home** (bool): If set save data from camera while moving.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## search-areas

**Description**: This node specifies a search of a set of area. The areas are 'closed' by assuming a segment between the last and first position in the list of geopoints. It is allowed to prepare for moving inside the node, for example doing take-off before flying.

**Tags**: vehicle

**TST Parameters**: None

**Task Parameters**:

- **areas** (geopointarrays): The areas to search.
- **speed** (string): Qualitative speed level.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.
- **target-type** (string): The type of the search target. Possible values: human, boat, ship, car, ...
- **area-type** (string): The type of the search area given as a list of tags. Possible tag values: water, beach, forrest, field
- **target-size** (float64): The estimates size of the search target in meter.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## search-around-position

**Description**: This node specifies a search of an area given by a list of geopoints. It is allowed to prepare for moving inside the node, for example doing take-off before flying.

**Tags**: vehicle

**TST Parameters**: None

**Task Parameters**:

- **geopoint** (geopoint): The area to search.
- **radius** (float64): The radius around the geopoint to focus on.
- **speed** (string): Qualitative speed level.  Possible values are 'fast', 'standard' and 'slow', and the meaning of these parameters is platform-dependent.
- **target-type** (string): The type of the search target. Possible values: person, boat, ship, car, ...
- **target-size** (float64): The estimates size of the search target in meter.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: None

## sit

**Description**: Causes the platform to land.

**Tags**: vehicle

**Task Parameters**: None

**Node type**: terminal

**Required params for execution**: None

**Node resources**: move

## stand

**Description**: Causes the platform to land.

**Tags**: vehicle

**Task Parameters**: None

**Node type**: terminal

**Required params for execution**: None

**Node resources**: move

## start-data-recording

**Description**: This node starts recording all the topics in the topics parameter. If topics does not start with a / then the namespace is added.

**Tags**: None

**Parameters**:

- **options** (strings): options for the recording, such as compression (optional, if set, should have the same length as topics).

**Node type**: terminal

**Required params for execution**: topics dataset-uri

**Node resources**: None

## stop-data-recording

**Description**: This node stops recording all the ROS topics in the topics parameter.  If a topic have been started twice you have to stop it twice also to really stop the recording.

**Tags**: None

**Parameters**:


**Node type**: terminal

**Required params for execution**: topics dataset-uri

**Node resources**: None

## wait

**Description**: Wait for a specified amount of time.

**Tags**: None

**Parameters**:

- **duration** (int32): Time to wait in seconds. If negative wait forever.

**Node type**: terminal

**Required params for execution**: duration

**Node resources**: None

## yaw

**Description**: Causes the platform to yaw and point in the specified direction.

**Tags**: vehicle

**Parameters**:

- **heading** (float64): The desired yaw angle in degrees.  0 is north, and positive clockwise.
- **yaw-rate** (string): A qualitative specification of the desired yaw-rate. The meaning is platform dependent. Possible values: fast, standard, slow
- **p** (geopoint): The position to yaw towards if the flag is set.

**Node type**: terminal

**Required params for execution**: None

**Node resources**: fly

